@@ -3,6 +3,8 @@ use crate::app::HelpMenuLine;
 use crate::app::NodeFilter;
 use crate::app::NodeSorter;
 use crate::app::NodeSorterApplicable;
+use crate::node::Node;
+use crate::search::SearchAlgorithm;
 use crate::ui::Border;
 use crate::ui::BorderType;
 use crate::ui::Constraint;
@@ -80,6 +82,40 @@ pub struct NodeTypesConfig {
     pub special: HashMap<String, NodeTypeConfig>,
 }
 
+impl NodeTypesConfig {
+    pub fn get(&self, node: &Node) -> NodeTypeConfig {
+        let mut node_type = if node.is_symlink {
+            self.symlink.to_owned()
+        } else if node.is_dir {
+            self.directory.to_owned()
+        } else {
+            self.file.to_owned()
+        };
+
+        let mut me = node.mime_essence.splitn(2, '/');
+        let mimetype: String = me.next().map(|s| s.into()).unwrap_or_default();
+        let mimesub: String = me.next().map(|s| s.into()).unwrap_or_default();
+
+        if let Some(conf) = self
+            .mime_essence
+            .get(&mimetype)
+            .and_then(|t| t.get(&mimesub).or_else(|| t.get("*")))
+        {
+            node_type = node_type.extend(conf);
+        }
+
+        if let Some(conf) = self.extension.get(&node.extension) {
+            node_type = node_type.extend(conf);
+        }
+
+        if let Some(conf) = self.special.get(&node.relative_path) {
+            node_type = node_type.extend(conf);
+        }
+
+        node_type
+    }
+}
+
 #[derive(Debug, Clone, Default, PartialEq, Eq, Serialize, Deserialize)]
 #[serde(deny_unknown_fields)]
 pub struct UiConfig {
@@ -146,6 +182,23 @@ pub struct TableConfig {
     pub col_widths: Option<Vec<Constraint>>,
 }
 
+#[derive(Debug, Clone, Default, Serialize, Deserialize)]
+#[serde(deny_unknown_fields)]
+pub struct SelectionConfig {
+    #[serde(default)]
+    pub item: UiElement,
+}
+
+#[derive(Debug, Clone, Default, Serialize, Deserialize)]
+#[serde(deny_unknown_fields)]
+pub struct SearchConfig {
+    #[serde(default)]
+    pub algorithm: SearchAlgorithm,
+
+    #[serde(default)]
+    pub unordered: bool,
+}
+
 #[derive(Debug, Clone, Default, Serialize, Deserialize)]
 #[serde(deny_unknown_fields)]
 pub struct LogsConfig {
@@ -172,6 +225,16 @@ pub struct SortDirectionIdentifiersUi {
     pub reverse: UiElement,
 }
 
+#[derive(Debug, Clone, Default, Serialize, Deserialize)]
+#[serde(deny_unknown_fields)]
+pub struct SearchDirectionIdentifiersUi {
+    #[serde(default)]
+    pub ordered: UiElement,
+
+    #[serde(default)]
+    pub unordered: UiElement,
+}
+
 #[derive(Debug, Clone, Default, Serialize, Deserialize)]
 #[serde(deny_unknown_fields)]
 pub struct SortAndFilterUi {
@@ -191,7 +254,10 @@ pub struct SortAndFilterUi {
     pub filter_identifiers: HashMap<NodeFilter, UiElement>,
 
     #[serde(default)]
-    pub search_identifier: Option<UiElement>,
+    pub search_direction_identifiers: SearchDirectionIdentifiersUi,
+
+    #[serde(default)]
+    pub search_identifiers: HashMap<SearchAlgorithm, UiElement>,
 }
 
 #[derive(Debug, Clone, Default, Serialize, Deserialize)]
@@ -249,6 +315,12 @@ pub struct GeneralConfig {
     #[serde(default)]
     pub table: TableConfig,
 
+    #[serde(default)]
+    pub selection: SelectionConfig,
+
+    #[serde(default)]
+    pub search: SearchConfig,
+
     #[serde(default)]
     pub default_ui: UiConfig,
 
